library example.basic_auth.server;

import 'dart:async';
import 'package:jaguar/jaguar.dart';
import 'package:jaguar_reflect/jaguar_reflect.dart';
import 'package:jaguar_auth/jaguar_auth.dart';
import 'package:jaguar_serializer/jaguar_serializer.dart';
import 'package:jaguar_json/jaguar_json.dart';

import '../../model/model.dart';
import '../../model/auth_model_manager.dart';

final Map<String, User> kUsers = {
  '0': new User('0', 'teja', 'word'),
  '1': new User('1', 'kleak', 'pass'),
};

final WhiteListPasswordChecker kModelManager =
    new WhiteListPasswordChecker(kUsers);

final Map<String, Book> _books = {
  '0': new Book.make('0', 'Book0', 'Author0'),
  '1': new Book.make('1', 'Book1', 'Author1'),
  '2': new Book.make('2', 'Book2', 'Author2'),
};

/// This route group contains login and logout routes
@Api()
class AuthRoutes extends Object with JsonRoutes {
  JsonRepo get repo => jsonRepo;

  @Post(path: '/login')
  @WrapOne(#formAuth)
  Response<String> login(Context ctx) {
    final User user = ctx.getInterceptorResult<User>(FormAuth);
    return toJson(user);
  }

  @Post(path: '/logout')
  Future logout(Context ctx) async {
    // Clear session data
    (await ctx.req.session).clear();
  }

  FormAuth formAuth(Context ctx) => new FormAuth(kModelManager);
}

/// Collection of routes students can also access
@Api(path: '/book')
@WrapOne(#authorizer)
class StudentRoutes extends Object with JsonRoutes {
  JsonRepo get repo => jsonRepo;

  @Get()
  Response<String> getAllBooks(Context ctx) {
    return toJson(_books.values);
  }

  @Get(path: '/:id')
  Response<String> getBook(Context ctx) {
    String id = ctx.pathParams.get('id');
    Book book = _books[id];
    return toJson(book);
  }

  Authorizer authorizer(Context ctx) => new Authorizer(kModelManager);
}

@Api(path: '/api')
class LibraryApi {
  @IncludeApi()
  final auth = new AuthRoutes();

  @IncludeApi()
  final student = new StudentRoutes();
}

server() async {
  final server = new Jaguar(port: 10000);
  server.addApi(reflect(new LibraryApi()));
  await server.serve();
}
